"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchemaHash = void 0;
const common_1 = require("./common");
const language_1 = require("graphql/language");
const execution_1 = require("graphql/execution");
const utilities_1 = require("graphql/utilities");
const fast_json_stable_stringify_1 = __importDefault(require("fast-json-stable-stringify"));
const crypto_1 = require("crypto");
function generateSchemaHash(schema) {
    return __awaiter(this, void 0, void 0, function* () {
        const introspectionQuery = utilities_1.getIntrospectionQuery();
        const documentAST = language_1.parse(introspectionQuery);
        const result = yield execution_1.execute(schema, documentAST);
        if (!result || !result.data || !result.data.__schema) {
            throw new Error(`${common_1.pluginName}: Unable to generate server introspection document.`);
        }
        const introspectionSchema = result.data.__schema;
        const stringifiedSchema = fast_json_stable_stringify_1.default(introspectionSchema);
        return crypto_1.createHash('sha512')
            .update(stringifiedSchema)
            .digest('hex');
    });
}
exports.generateSchemaHash = generateSchemaHash;
//# sourceMappingURL=schema.js.map